@@ -22,11 +22,10 @@ import logging
 import os
 
 from PyQt5.QtCore import (pyqtSignal, QObject, QProcess)
-from PyQt5.QtWidgets import QApplication
 
 from pre_workbench.bbuf_parsing import apply_grammar_on_bbuf
 from pre_workbench.configs import SettingsField, SettingsSection, registerOption, getValue
-from pre_workbench.guihelper import qApp
+from pre_workbench.guihelper import APP
 from pre_workbench.objects import ByteBuffer, ByteBufferList, ReloadRequired
 from pre_workbench.structinfo.exceptions import invalid, incomplete
 from pre_workbench.structinfo.pcap_reader import read_pcap_file, PcapFormats
@@ -42,6 +41,7 @@ registerOption(group, "tsharkBinary", "tshark Binary", "text", {"fileselect":"op
 
 DataSourceTypes = TypeRegistry()
 
+
 class DataSource(QObject):
 	on_finished = pyqtSignal()
 	logger = logging.getLogger("DataSource")
@@ -55,6 +55,7 @@ class DataSource(QObject):
 	def cancelFetch(self):
 		pass
 
+
 class SyncDataSource(DataSource):
 	def startFetch(self):
 		obj = self.process()
@@ -64,6 +65,17 @@ class SyncDataSource(DataSource):
 	def cancelFetch(self):
 		pass
 
+
+class MacroDataSource(SyncDataSource):
+	def __init__(self, macro_container_id, macroname, params):
+		super().__init__(params)
+		self.macro_container_id = macro_container_id
+		self.macroname = macroname
+	def process(self):
+		macro = APP().get_macro(self.macro_container_id, self.macroname)
+		return macro.execute(None, self.params)
+
+
 @DataSourceTypes.register(DisplayName="Binary file")
 class FileDataSource(SyncDataSource):
 	@staticmethod
@@ -320,5 +332,5 @@ class LivePcapCaptureDataSource(DataSource):
 		self.process.kill()
 
 def formatinfoSelect(dialog):
-	names = QApplication.instance().project.formatInfoContainer.definitions.keys()
+	names = APP().project.formatInfoContainer.definitions.keys()
 	return zip(names, names)
