@@ -23,20 +23,22 @@ import os
 import sys
 import traceback
 import uuid
+from glob import glob
 
 from PyQt5.QtCore import (Qt, pyqtSlot, QSignalMapper, QTimer, pyqtSignal, QUrl)
 from PyQt5.QtGui import QKeySequence, QColor, QDesktopServices
 from PyQt5.QtWidgets import QMainWindow, QAction, QFileDialog, QWidget, QMessageBox, QToolButton, QLabel, QApplication, \
-	QPushButton
+	QPushButton, QTextBrowser, QDialogButtonBox
 from PyQtAds import ads
 
+from pre_workbench.consts import WDGEN_HELP_URL
 from pre_workbench.errorhandler import check_for_updates
 from pre_workbench.guihelper import navigateBrowser, TODO, APP, showWidgetDlg
 from pre_workbench.app import NavigateCommands, GlobalEvents
 from pre_workbench import configs, consts
 # noinspection PyUnresolvedReferences
 from pre_workbench import windows
-from pre_workbench.configs import getIcon, SettingsSection
+from pre_workbench.configs import getIcon, SettingsSection, SettingsField, icon_searchpaths
 from pre_workbench.datawidgets import DynamicDataWidget
 from pre_workbench.util import get_app_version, SimpleThread
 from pre_workbench.windows.dialogs.manageannotationsets import ManageAnnotationSetsDialog
@@ -44,7 +46,8 @@ from pre_workbench.windows.dockwindows import FileBrowserWidget, MdiWindowListWi
 	StructInfoCodeWidget, DataInspectorWidget, BinwalkDockWidget, ExtToolDockWidget, SearchDockWidget, \
 	MacroListDockWidget
 from pre_workbench.windows.dockwindows import RangeTreeDockWidget, RangeListWidget, SelectionHeuristicsConfigWidget, LogWidget
-from pre_workbench.controls.genericwidgets import MemoryUsageWidget, showPreferencesDlg
+from pre_workbench.controls.genericwidgets import MemoryUsageWidget, showPreferencesDlg, showListSelectDialog, \
+	showSettingsDlg
 from pre_workbench.typeeditor import JsonView
 from pre_workbench.typeregistry import WindowTypes, DockWidgetTypes
 from pre_workbench.windows.content.hexfile import HexFileWindow
@@ -66,6 +69,7 @@ class WorkbenchMain(QMainWindow):
 		self.activeDocumentMeta = dict()
 		self.activeDocumentWindow = None
 		self._initUI()
+		APP().update_splash("Restoring windows...")
 		self.restoreChildren()
 		self.mdiArea.restoreState(self.project.getValue("DockState", b""))
 
@@ -228,9 +232,7 @@ class WorkbenchMain(QMainWindow):
 
 		##### FILE #####
 		fileMenu = menubar.addMenu('&File')
-		newTbAct = QToolButton(self, icon=getIcon('document--plus.png'), text='New', popupMode=QToolButton.InstantPopup)
 		newTbMenu = fileMenu.addMenu("New Tab")
-		newTbAct.setMenu(newTbMenu)
 		for wndTyp, meta in WindowTypes.types:
 			text = 'New '+meta.get('displayName', meta.get('name'))
 			newAct = QAction(text, self, statusTip=text,
@@ -239,7 +241,6 @@ class WorkbenchMain(QMainWindow):
 			if wndTyp.__name__ == 'ObjectWindow': newAct.setShortcut('Ctrl+N')
 			#fileMenu.addAction(newAct)
 			newTbMenu.addAction(newAct)
-		mainToolbar.addWidget(newTbAct)
 		#fileMenu.addAction(self.newAct)
 
 		fileMenu.addAction(self.openAct)
@@ -247,9 +248,8 @@ class WorkbenchMain(QMainWindow):
 		fileMenu.addAction(self.saveAsAct)
 		fileMenu.addAction(self.reloadFileAct)
 		fileMenu.addSeparator()
-		exportMenu = fileMenu.addMenu('Export')
-		exportMenu.addAction("As Python Script")
-		exportMenu.addAction("As Wireshark Lua Dissector")
+		#exportMenu = fileMenu.addMenu('&Export')
+		#exportMenu.addAction("As &Python Script")
 		fileMenu.addSeparator()
 
 		fileMenu.addAction(self.loadProjectAct)
@@ -288,7 +288,7 @@ class WorkbenchMain(QMainWindow):
 		##### Annotations #####
 		annotationsMenu = menubar.addMenu('&Annotations')
 		annotationsMenu.addAction(self.clearRangesAct)
-		self.loadAnnotationSetMenu = annotationsMenu.addMenu("Load Annotation Set")
+		self.loadAnnotationSetMenu = annotationsMenu.addMenu("Load &Annotation Set")
 		annotationsMenu.aboutToShow.connect(self._updateParserMenu)
 		annotationsMenu.addAction(self.manageAnnotationSetsAct)
 
@@ -296,19 +296,23 @@ class WorkbenchMain(QMainWindow):
 		##### PARSER #####
 		parserMenu = menubar.addMenu('&Parser')
 		parserMenu.addAction(self.reloadGrammarAct)
-		self.applyFormatInfoMenu = parserMenu.addMenu("Parse Buffer")
+		self.applyFormatInfoMenu = parserMenu.addMenu("&Parse Buffer")
 		parserMenu.aboutToShow.connect(self._updateParserMenu)
+		parserMenu.addAction(QAction("Export As Wireshark &Lua Dissector", self, triggered=self.onWiresharkExport))
 
 		##### TOOLS #####
 		toolsMenu = menubar.addMenu('&Tools')
-		showConfigAction = QAction("Show config", self, triggered=lambda: self.showChild(JsonView(jdata=configs.configDict)),
+		self.macroMenu = toolsMenu.addMenu('&Run Macro')
+		self.macroMenu.aboutToShow.connect(self._updateMacroMenu)
+		showConfigAction = QAction("Show Raw &Config", self, triggered=lambda: self.showChild(JsonView(jdata=configs.configDict)),
 								   shortcut='Ctrl+Shift+,')
 		toolsMenu.addAction(showConfigAction)
-		editConfigAction = QAction(getIcon("wrench-screwdriver.png"), "Preferences ...", self, triggered=lambda: self.onPreferences(),
+		editConfigAction = QAction(getIcon("wrench-screwdriver.png"), "&Preferences ...", self, triggered=lambda: self.onPreferences(),
 								   menuRole=QAction.PreferencesRole, shortcut='Ctrl+,')
 		toolsMenu.addAction(editConfigAction)
 		if APP().plugins_dir:
-			toolsMenu.addAction(QAction("Open Plugins Dir", self, triggered=lambda: QDesktopServices.openUrl(QUrl.fromLocalFile(APP().plugins_dir))))
+			toolsMenu.addAction(QAction("&Open Plugins Directory", self, triggered=lambda: QDesktopServices.openUrl(QUrl.fromLocalFile(APP().plugins_dir))))
+			toolsMenu.addAction(QAction("&Manage Plugins", self, triggered=self.onManagePlugins))
 
 		##### WINDOW #####
 		self.windowMenu = menubar.addMenu("&Window")
@@ -317,24 +321,32 @@ class WorkbenchMain(QMainWindow):
 
 		##### HELP #####
 		helpMenu = menubar.addMenu("&Help")
-		helpMenu.addAction(QAction("Getting started", self, triggered=lambda: navigateBrowser(consts.GETTING_STARTED_URL)))
-		helpMenu.addAction(QAction("Syntax reference", self, triggered=lambda: navigateBrowser(consts.SYNTAX_REFERENCE_URL)))
-		helpMenu.addAction(QAction("Key bindings", self, triggered=lambda: navigateBrowser(consts.KEY_BINDINGS_URL)))
+		helpMenu.addAction(QAction("Getting Started", self, triggered=lambda: navigateBrowser(consts.GETTING_STARTED_URL)))
+		helpMenu.addAction(QAction("Syntax Reference", self, triggered=lambda: navigateBrowser(consts.SYNTAX_REFERENCE_URL)))
+		helpMenu.addAction(QAction("Key Bindings", self, triggered=lambda: navigateBrowser(consts.KEY_BINDINGS_URL)))
 
-		helpMenu.addAction(QAction("Issue tracker", self, triggered=lambda: navigateBrowser(consts.ISSUE_TRACKER_URL)))
+		helpMenu.addAction(QAction("Issue Tracker", self, triggered=lambda: navigateBrowser(consts.ISSUE_TRACKER_URL)))
 		helpMenu.addSeparator()
 		helpMenu.addAction(QAction("About PRE Workbench", self, triggered=lambda: self.showAboutBox(),
 								   menuRole=QAction.AboutRole))
 
+		##### MAIN TOOLBAR #####
+		newTbAct = QToolButton(self, icon=getIcon('document--plus.png'), text='New', popupMode=QToolButton.InstantPopup)
+		newTbAct.setMenu(newTbMenu)
+		mainToolbar.addWidget(newTbAct)
 		mainToolbar.addAction(self.openAct)
 		mainToolbar.addAction(self.saveAct)
 		mainToolbar.addAction(self.saveAsAct)
 		mainToolbar.addAction(editConfigAction)
+		runMacroAction = QToolButton(self, icon=getIcon("scripts.png"), text="Run Macro", popupMode=QToolButton.InstantPopup)
+		runMacroAction.setMenu(self.macroMenu)
+		mainToolbar.addWidget(runMacroAction)
 
 	def _initDockWindows(self):
 		self.dockWidgets = {}
 
 		self.zoomWindow = DynamicDataWidget()
+		self.zoomWindow.block_zoom = True # don't forward zoom events from inside the zoom window - causes app crashes
 		self.zoomWindow.meta_updated.connect(self.onMetaUpdateRaw)
 		self.createDockWnd("Zoom", "Zoom", "document-search-result.png", self.zoomWindow, ads.BottomDockWidgetArea, showFirstRun=True)
 		self.zoom_updated.connect(lambda content: self.zoomWindow.setContents(content) if content is not None else None)
@@ -401,14 +413,22 @@ class WorkbenchMain(QMainWindow):
 		self.show()
 
 	def showAboutBox(self):
-		QMessageBox.about(self, "PRE Workbench", "Protocol Reverse Engineering Workbench\n"
-												 "Version " + get_app_version() +"\n\n"
-												 "Copyright (c) 2022 Mira Weller\n"
-												 "Licensed under the GNU General Public License Version 3\n\n"
-												 + consts.WEBSITE_URL + "\n\n"
-						  "Config Dir: " + configs.dirs.user_config_dir + "\n"
-						  "Plugins Dir: " + APP().plugins_dir + "\n" +
-						  ("Loaded Plugins:\n" + "\n".join(APP().plugins.keys()) if len(APP().plugins) > 0 else ""))
+		from pre_workbench import errorhandler
+		showWidgetDlg(QTextBrowser(minimumWidth=520,minimumHeight=300,searchPaths=icon_searchpaths,openLinks=False,anchorClicked=navigateBrowser,
+							html="<img src='appicon.png' style='float: left; '><br><b>Protocol Reverse Engineering Workbench</b><br>"
+							"Version " + get_app_version() +"<br><br>"
+							"Copyright (c) 2022 Mira Weller<br>"
+							"Licensed under the GNU General Public License Version 3<br style='clear:left'><br><hr>"
+							"<table>"
+							"<tr><td>Website: </td><td><a href='"+ consts.WEBSITE_URL + "'>"+ consts.WEBSITE_URL + "</a></td></tr>"
+							"<tr><td>Issue Tracker: </td><td><a href='"+ consts.ISSUE_TRACKER_URL + "'>"+ consts.ISSUE_TRACKER_URL + "</a></td></tr>"
+							"<tr><td>&nbsp;</td><td></td></tr>"
+							"<tr><td>Config Dir: </td><td><a href='file:" + configs.dirs.user_config_dir + "'>" + configs.dirs.user_config_dir + "</a></td></tr>"
+							"<tr><td>Log File: </td><td>" + errorhandler.logFile + "</td></tr>"
+							"<tr><td>Plugins Dir: </td><td><a href='file:" + APP().plugins_dir + "'>" + APP().plugins_dir + "</a></td></tr>" +
+							("<tr><td>Loaded Plugins:</td><td>" + "<br>".join(APP().plugins.keys())+"</td></tr>" if len(APP().plugins) > 0 else "")
+							+ "</table>"),
+					  "About PRE Workbench", lambda: None, self, buttons=QDialogButtonBox.Close)
 
 	def onPreferences(self):
 		res = showPreferencesDlg(configs.configDefinitions, configs.configDict, "Preferences", self)
@@ -417,6 +437,31 @@ class WorkbenchMain(QMainWindow):
 				configs.setValue(k,v)
 		GlobalEvents.on_config_change.emit()
 
+	def onManagePlugins(self):
+		plugins = ["pre_workbench.plugins." + os.path.basename(file)[:-3]
+				   for file in glob(os.path.join(APP().plugins_dir, "*.py"))]
+		enabled_plugins = configs.getValue("EnabledPlugins", [])
+		result = showListSelectDialog(list(zip(plugins, plugins)), enabled_plugins,
+									  title="Manage Plugins - Select To Enable",
+									  parent=self, multiselect=True, min_width=400)
+		if result is not None:
+			configs.setValue("EnabledPlugins", result)
+
+	def onWiresharkExport(self):
+		from pre_workbench.datasource import formatinfoSelect
+		r = showSettingsDlg([
+			SettingsField("definition", "Start Grammar Definition", "text", {"listselectcallback":formatinfoSelect}),
+			SettingsField("only_types", "Only Export These Definitions", "text", {"listselectcallback":formatinfoSelect,"multiselect":","}),
+			SettingsField("dissector_table", "Dissector Table", "text", {}),
+			SettingsField("output_file", "Output File", "text", {"fileselect":"save","filter":"Lua Script (*.lua)"}),
+		], self.project.getValue("LastWiresharkExportOptions", {}), "Wireshark Dissector Export", self, min_width=550,
+		help_callback=lambda: navigateBrowser(WDGEN_HELP_URL))
+		if r:
+			from pre_workbench.wdgen.lua import generate_lua_dissector
+			self.project.setValue("LastWiresharkExportOptions", r)
+			with open(r['output_file'], 'w') as out:
+				generate_lua_dissector(r['definition'], r['only_types'], r['dissector_table'].split(','), self.project.formatInfoContainer, out)
+
 	def onMruClicked(self):
 		self.openFile(self.sender().data())
 
@@ -465,6 +510,11 @@ class WorkbenchMain(QMainWindow):
 			self.applyFormatInfoMenu.addAction(name, TODO)
 		#menu.addAction("New...", lambda: )
 
+	def _updateMacroMenu(self):
+		self.macroMenu.clear()
+		for container_id, container, macroName in APP().find_macros_by_input_types(["NONE"]):
+			self.macroMenu.addAction(macroName, lambda c=container, name=macroName: c.getMacro(name).execute(None))
+
 	def onSubWindowActivated(self, old: ads.CDockWidget, now: ads.CDockWidget):
 		logging.log(logging.TRACE, "onSubWindowActivated")
 		self.updateMappedChildActions()
