@@ -19,6 +19,7 @@ import time
 import traceback
 
 from PyQt5 import QtGui
+from PyQt5 import QtCore
 from PyQt5.QtCore import (Qt)
 from PyQt5.QtGui import QColor
 from PyQt5.QtWidgets import QMenu, QFileDialog, QTreeWidget, QTreeWidgetItem, \
@@ -29,14 +30,20 @@ from pre_workbench.algo.range import Range
 from pre_workbench.configs import SettingsField
 from pre_workbench.controls.genericwidgets import showSettingsDlg
 from pre_workbench.controls.scintillaedit import showScintillaDialog
+from pre_workbench.guihelper import getMonospaceFont, filledColorIcon, setClipboardText
 from pre_workbench.interactive_fic import InteractiveFormatInfoContainer
 from pre_workbench.structinfo.format_info import FormatInfo, StructFI, VariantStructFI, SwitchFI, RepeatStructFI, \
 	UnionFI, BitStructFI
 from pre_workbench.typeeditor import showTypeEditorDlg, showTreeEditorDlg
-from pre_workbench.util import PerfTimer
+from pre_workbench.util import PerfTimer, truncate_str
 
 
 class RangeTreeWidget(QTreeWidget):
+	RangeRole = QtCore.Qt.UserRole
+	BytesOffsetRole = QtCore.Qt.UserRole + 1
+	BytesSizeRole = QtCore.Qt.UserRole + 2
+	SourceDescRole = QtCore.Qt.UserRole + 3
+
 	def __init__(self, parent=None):
 		super().__init__(parent)
 		self.itemActivated.connect(self._fiTreeItemActivated)
@@ -53,6 +60,8 @@ class RangeTreeWidget(QTreeWidget):
 		self.header().moveSection(5, 3)
 		self.setContextMenuPolicy(Qt.CustomContextMenu)
 		self.customContextMenuRequested.connect(self._onCustomContextMenuRequested)
+		self.header().setContextMenuPolicy(Qt.CustomContextMenu)
+		self.header().customContextMenuRequested.connect(self._onHeaderCustomContextMenuRequested)
 		self.formatInfoContainer = None
 		self.optionsConfigKey = "RangeTree"
 		self.setMouseTracking(True)
@@ -67,11 +76,7 @@ class RangeTreeWidget(QTreeWidget):
 		self.clear()
 		for fi_tree in fi_trees:
 			if fi_tree is None: continue
-			root = QTreeWidgetItem(self)
-			root.setExpanded(True)
-			if self.formatInfoContainer:
-				root.setText(0, self.formatInfoContainer.file_name)
-			fi_tree.addToTree(root, self.onlyPrintable)
+			Range_addToTree(fi_tree, self, self.onlyPrintable)
 
 	def _fiTreeItemActivated(self, item, column):
 		pass
@@ -83,7 +88,7 @@ class RangeTreeWidget(QTreeWidget):
 		self.setUpdatesEnabled(False)
 		while iterator.value():
 			item = iterator.value()
-			itemRange = item.data(0, Range.RangeRole)
+			itemRange = item.data(0, RangeTreeWidget.RangeRole)
 			bgColor = QColor("#dddddd") if itemRange is not None and itemRange.overlaps(range) else QColor("#ffffff")
 			with PerfTimer("setBackground"):
 				item.setBackground(0, bgColor)
@@ -100,13 +105,11 @@ class RangeTreeWidget(QTreeWidget):
 
 	def _onCustomContextMenuRequested(self, point):
 		ctx = QMenu("Context menu", self)
-		item = self.itemAt(point)
-		if item != None:
-			range = item.data(0, Range.RangeRole)
-			source = item.data(0, Range.SourceDescRole)
+		if item := self.itemAt(point):
+			range = item.data(0, RangeTreeWidget.RangeRole)
+			source = item.data(0, RangeTreeWidget.SourceDescRole)
 
-			if item.parent() != None:
-				parentSource = item.parent().data(0, Range.SourceDescRole)
+			parentSource = item.parent().data(0, RangeTreeWidget.SourceDescRole) if item.parent() else None
 			if isinstance(source, FormatInfo):
 				if isinstance(source.fi, (StructFI, UnionFI)):
 					ctx.addAction("Add field ...", lambda: self.addField(source, "StructField"))
@@ -129,20 +132,26 @@ class RangeTreeWidget(QTreeWidget):
 				for key, name, style in guihelper.getHighlightStyles():
 					ctx.addAction(name+"\t"+key, lambda style=style: self.styleSelection(source, **style))
 				ctx.addSeparator()
-		else:
-			ctx.addAction("Edit ...", lambda: self.editField(self.formatInfoContainer.definitions[self.formatInfoContainer.main_name]))
+			ctx.addAction("Copy", lambda: setClipboardText("\t".join(item.text(i) for i in range(item.columnCount()))))
+		ctx.addSeparator()
+		self._buildGenericContextMenu(ctx)
 
-		ctx.addAction("New grammar file ...", self.newFormatInfo)
-		ctx.addAction("Load grammar file ...", self.fileOpenFormatInfo)
-		if self.formatInfoContainer and self.formatInfoContainer.file_name:
-			ctx.addAction("Save grammar file", lambda: self.saveFormatInfo(self.formatInfoContainer.file_name))
-		ctx.addAction(QAction(parent=ctx, text="Only printable", triggered=self._toggleOnlyPrintable, checkable=True, checked=self.onlyPrintable))
 		ctx.exec(self.mapToGlobal(point))
 
+	def _onHeaderCustomContextMenuRequested(self, point):
+		ctx = QMenu("Context menu", self)
+		self._buildGenericContextMenu(ctx)
+		ctx.exec(self.mapToGlobal(point))
+
+	def _buildGenericContextMenu(self, ctx):
+		ctx.addAction("Expand All", lambda: self.expandAll())
+		ctx.addAction("Collapse All", lambda: self.collapseAll())
+		ctx.addAction(QAction(parent=ctx, text="Only printable", triggered=self._toggleOnlyPrintable, checkable=True, checked=self.onlyPrintable))
+
+
 	def _toggleOnlyPrintable(self):
 		self.onlyPrintable = not self.onlyPrintable
 
-
 	def addField(self, parent, typeName):
 		def ok(params):
 			parent.updateParams(children=parent.params['children']+[params])
@@ -214,8 +223,7 @@ class RangeTreeWidget(QTreeWidget):
 												  configs.getValue(self.optionsConfigKey + "_lastOpenFile", ""),
 												  "Grammar Files (*.pfi *.txt)")
 		if not fileName: return
-		self.formatInfoContainer = InteractiveFormatInfoContainer(self, )
-		self.formatInfoContainer.load_from_string("DEFAULT struct(endianness="<") {}")
+		self.formatInfoContainer = InteractiveFormatInfoContainer(load_from_string="DEFAULT struct(endianness="<") {}")
 		self.formatInfoContainer.file_name = fileName
 		self.saveFormatInfo(self.formatInfoContainer.file_name)
 
@@ -254,3 +262,59 @@ class RangeTreeWidget(QTreeWidget):
 
 	#endregion
 
+
+def Range_addToTree(range: Range, parent: QTreeWidgetItem, printableOnly: bool = False):
+	x = range
+	collapse = x.field_name.startswith("_")
+	text0 = x.field_name
+	text1 = str(x.start) + "+" + str(x.bytes_size)
+	text2 = str(x.source_desc)
+	color = x.metadata.get("color")
+	print = []
+	if x.metadata.get("print"): print.append(x.metadata["print"])
+	while type(x.value) == Range:
+		x = x.value
+		if x.field_name.startswith("_"): collapse = True
+		text0 += " >> " + x.field_name
+		text1 += " >> " + str(x.start) + "+" + str(x.bytes_size)
+		text2 += " >> " + str(x.source_desc)
+		if x.metadata.get("color"): color = x.metadata["color"]
+		if x.metadata.get("print"): print.append(x.metadata["print"])
+
+	if ((len(print) > 0 or not printableOnly) and not x.metadata.get("hide")) or x.exception is not None:
+		me = QTreeWidgetItem(parent)
+		me.setData(0, RangeTreeWidget.RangeRole, range)
+		me.setData(0, RangeTreeWidget.BytesOffsetRole, range.start)
+		me.setData(0, RangeTreeWidget.BytesSizeRole, range.bytes_size)
+		me.setData(0, RangeTreeWidget.SourceDescRole, x.source_desc)
+		me.setFont(5, getMonospaceFont())
+		me.setText(0, truncate_str(text0))
+		me.setText(1, truncate_str(text1))
+		me.setText(2, truncate_str(text2))
+		if x.exception is not None:
+			me.setForeground(3, QColor("red"))
+			me.setText(3, str(x.exception).split("\n", 1)[0])
+		elif type(x.value) == dict:
+			if not collapse: me.setExpanded(True)
+			for item in x.value.values():
+				Range_addToTree(item, me, printableOnly)
+		elif type(x.value) == list:
+			if not collapse:me.setExpanded(True)
+			for item in x.value:
+				Range_addToTree(item, me, printableOnly)
+		else:
+			try:
+				me.setText(3, truncate_str(x.source_desc.formatter(x.value)))
+			except:
+				me.setText(3, truncate_str(x.value))
+		if color:
+			me.setIcon(3, filledColorIcon(color, 16))
+		me.setText(5, " >> ".join(print))
+	else:
+		if type(x.value) == dict:
+			for item in x.value.values():
+				Range_addToTree(item, parent, printableOnly)
+		if type(x.value) == list:
+			for item in x.value:
+				Range_addToTree(item, parent, printableOnly)
+
